/*
*
*  BaseAction.java
*
*  SPECT: Simple Portal-Enhancing Configuration Tool
*  Copyright (C) 1999, 2000 Ray Ontko & Co.
*  http://www.ontko.com/
*
*  This program is a simple-to-use configuration tool for Brio.Portal
*  (formerly ReportMart).
*
*  "Brio.Portal" is a trademark of Brio Technology, http://www.brio.com/ .
* 
* 
*  This program is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or 
*  (at your option) any later version. 
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of 
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
*  Public License for more details. 
* 
*  A copy of the GPL is available online the GNU web site: 
* 
*    http://www.gnu.org/copyleft/gpl.html
* 
*/ 

import java.io.*;
import java.util.*;
import com.sqribe.rm.*;

/**********************************************************************
 **
 ** The superclass for all the action classes.  Provides methods for
 ** parsing the control file, populating the keyword hashtable, 
 ** logging, error handling, and parsing date and csv strings.
 **
 *********************************************************************/
public abstract class BaseAction {

  protected StreamTokenizer st;
  protected int action;
  protected String type;
  protected int lineno;
  protected Hashtable
    default_keywords,
    keywords;

  public BaseAction(int action, StreamTokenizer st, int lineno) 
         throws IOException {
    this.action = action;
    this.st = st;
    this.lineno = lineno;
  }

  public int parse(Hashtable default_keywords, Hashtable keywords) 
             throws IOException {
    this.keywords = keywords;
    this.default_keywords = default_keywords;
    RMProperty gp;
    st.nextToken();
    if (st.ttype == st.TT_WORD || st.ttype == '"') {
      gp = (RMProperty)keywords.get("NAME");
      gp.value = st.sval;
      gp.is_default = false;
      log("");
      log("Executing [" + lineno + "]: " + 
          RMLoader.decodeAction(action) + " " + 
          type + " \"" + gp.value + "\"");
    }
    else 
      RMLoader.error(st.lineno(), ": object name must be given.");
    while (true) {
      st.nextToken();
      if (st.ttype == ';' || st.ttype == st.TT_EOF)
        return 0;
      gp = (RMProperty)keywords.get(st.sval.toUpperCase());
      if (gp == null) {
        error("invalid keyword: " + st.sval);
        return 1;
      }
      if (st.nextToken() != '=') {
        error("No = for keyword " + gp.name);
        return 1;
      }
      st.nextToken();
      if (st.ttype == st.TT_WORD || st.ttype == '"' )
        gp.value = st.sval;
      else if (st.ttype == st.TT_NUMBER) 
        gp.value = String.valueOf((int)st.nval);
      else {
        error("unexpected token: " + (char)st.ttype);
        return 1;
      }
      gp.is_default = false;
    }
  } // public int parse()

  public abstract int doAction() ;


  /** Umask is a 3-character String representing an octal bitmask.
  ** We convert this to an integer, then loop through each bit
  ** and set the associated boolean in an array based on the bit's 
  ** value.  This array is then passed to the ReportMart API 
  ** Permission class's "new" method.  (Note that the lowest-order
  ** bit of the integer [the "1" bit] is associated with the
  ** boolean array element with the highest index [8], so the
  ** index i counts down toward the lower index as we examine
  ** higher- and higher-order bits.)
  */
  public Permissions convertUmask(String umask)
  {
                           
  boolean perms[] = new boolean[9];
  int umask_int = Integer.valueOf(umask,8).intValue();

  for (int i = perms.length - 1; i >= 0 ; --i)
  {
    perms[i] = (umask_int & 1) == 1;    // is the "ones" bit set?
    umask_int >>= 1;                    // shift one bit down (divide by 2)
    // System.out.println("perms[" + i + "]=" + perms[i]);
  }

  return(new Permissions(perms));
                                   
  } // end convertUmask


  /**
  *
  * Converts a string to a boolean.  Returns true if the string is
  * (case insensitive) "t", "true", "y", or "yes".  All other string
  * values will return false.
  *
  **/
  public boolean decodeBoolean(String value) {
    return (value.equalsIgnoreCase("Y") ||
            value.equalsIgnoreCase("YES") ||
            value.equalsIgnoreCase("T") ||
            value.equalsIgnoreCase("TRUE"));
  }


  /**
  *  PRE:  Takes a string which is either "none" or
  *        "+x" where x is the number of days from
  *        the current date/time from which this object is
  *        to expire from ReportMart
  *  POST: Returns a null value for "no expiration" or a
  *        Date object which is 'x' days from the present
  **/
   public Date decodeDate(String exp_string) {
     if(exp_string.length() == 0 || exp_string.equalsIgnoreCase("none"))
        return null;
     if (exp_string.charAt(0) == '+') {
       String exp_days = exp_string.substring(1);
       // create a Calendar with the default time zone
       // and the current date and time
       Calendar cal = Calendar.getInstance();
       cal.add(Calendar.DATE, Integer.decode(exp_days).intValue());
       return cal.getTime();
     }
     else {
       String date_mask = "yyyyMMddHHmmss";
       switch (exp_string.length()) {
       case 12 :
         break;
       case 10 :
         date_mask = date_mask.substring(0,10);
         break;
       case  8 :
         date_mask = date_mask.substring(0,8);
         break;
       default :
         error("invalid date format: " + exp_string);
         return null;
       }
       return null;
     }
   }


  /**
  *
  *  Parses a string by ',' into an array of strings.  
  *
  **/
  public String[] parseCommaSeparatedList(String keywds) {
    StringTokenizer stk = new StringTokenizer(keywds,",");
    String s[] = new String[stk.countTokens()];
    for (int i = 0; stk.hasMoreTokens(); i++)
      s[i] = stk.nextToken().trim();
    return s;
  }

  /** 
  *
  *  Searches the query vector for the newest element and returns it.
  *  this is used to when the config option WHEN-MULTIPLE-EXISTS is set
  *  to NEWEST.
  *
  **/
  protected BaseObject findNewestObject(QueryVector qv) {
    BaseObject bo = null;
    for (int i = 0; i < qv.size(); i++) {
      if (bo == null || bo.getDateCreated().before(
                        ((BaseObject)qv.elementAt(i)).getDateCreated()))
        bo = (BaseObject)qv.elementAt(i);
    }
    return bo;
  }

  /** 
  *
  *  Searches the query vector for the oldest element and returns it.
  *  this is used to when the config option WHEN-MULTIPLE-EXISTS is set
  *  to OLDEST.
  *
  **/
  protected BaseObject findOldestObject(QueryVector qv) {
    BaseObject bo = null;
    for (int i = 0; i < qv.size(); i++) {
      if (bo == null || bo.getDateCreated().after(
                        ((BaseObject)qv.elementAt(i)).getDateCreated()))
        bo = (BaseObject)qv.elementAt(i);
    }
    return bo;
  }

  /**
  *
  *  Checks to see if the given RMProperty has the default value.  If not,
  *  it checks the config option WHEN-IGNORED-KEYWORD and logs the appropriate
  *  message (if any is required).  The return value indicates that , 
  *  according to the config option, a fatal error has occurred.
  * 
  **/
  protected boolean keywordIgnored(RMProperty rmp) {
    if (!rmp.is_default) {
      switch (RMLoader.config.when_ignored_keyword) {
      case RMConfig.ERROR :
        error("ignored keyword, " + rmp.name);
        return true;
      case RMConfig.WARNING :
        warning("ignored keyword, " + rmp.name);
      case RMConfig.IGNORE :
      default :
        return false;
      }
    }
    else return false;
  }

  /**
  *
  *  Checks to see if any keywords (besides NAME) have non-default values.
  *  Returns true if any keyword causes keywordIgnored() to error.
  *
  **/
  protected boolean anyKeywordIgnored() {
    Enumeration e = keywords.elements();
    boolean return_status = false;
    while (e.hasMoreElements()) {
      RMProperty rmp = (RMProperty)e.nextElement();
      if (!rmp.name.equals("NAME"))
        return_status = return_status || keywordIgnored(rmp);
    }
    return return_status;
  }

  protected void printGoodRecord() {
    printRecord(RMLoader.good);
  }

  protected void printBadRecord() {
    printRecord(RMLoader.bad);
  }

  /**
  *
  *  Prints the record (the action, object type, name, and all the keyword/
  *  value pairs) to the given PrintWriter, usually either the good or
  *  bad file.
  *
  **/
  protected void printRecord(PrintWriter out) {
    if (out != null) {
      out.println("# Action started line [" + lineno + "]");
      if (keywords == null) System.out.println("keywords null!!!");
      RMProperty rmp = (RMProperty)keywords.get("NAME");
      if (rmp == null) System.out.println("rmp null!!!");
      out.println(RMLoader.decodeAction(action) + " " + type + " \"" + 
        rmp.value + "\"");
      Enumeration e = keywords.elements();
      while (e.hasMoreElements()) {
        rmp = (RMProperty)e.nextElement();
        if (!rmp.is_default && !rmp.name.equals("NAME"))
          out.println(rmp);
      }
      out.println(";");
      out.println("");
    }
  }

  protected void warning(String msg) {
    RMLoader.logMsg("WARNING [" + lineno + "]: " + msg);
  }

  protected void error(String msg) {
    RMLoader.logMsg("ERROR [" + lineno + "]: " + msg);
  }
 
  protected void log(String msg) {
    RMLoader.logMsg(msg);
  }
}
